@@ -1,5 +1,6 @@
 #include "commands.h"
 #include "exceptions.h"
+#include "git-version.h"
 #include "lite_operations.h"
 #include "myutils.h"
 #include "operations.h"
@@ -12,6 +13,7 @@
 #include <fuse.h>
 #include <json/json.h>
 #include <tclap/CmdLine.h>
+#include <utf8proc/utf8proc.h>
 
 #include <algorithm>
 #include <memory>
@@ -35,14 +37,14 @@ using namespace securefs;
 namespace
 {
 
-static const char* CONFIG_FILE_NAME = ".securefs.json";
-static const unsigned MIN_ITERATIONS = 20000;
-static const unsigned MIN_DERIVE_SECONDS = 1;
-static const size_t CONFIG_IV_LENGTH = 32, CONFIG_MAC_LENGTH = 16;
-static const char* PBKDF_ALGO_PKCS5 = "pkcs5-pbkdf2-hmac-sha256";
-static const char* PBKDF_ALGO_SCRYPT = "scrypt";
+const char* CONFIG_FILE_NAME = ".securefs.json";
+const unsigned MIN_ITERATIONS = 20000;
+const unsigned MIN_DERIVE_SECONDS = 1;
+const size_t CONFIG_IV_LENGTH = 32, CONFIG_MAC_LENGTH = 16;
+const char* PBKDF_ALGO_PKCS5 = "pkcs5-pbkdf2-hmac-sha256";
+const char* PBKDF_ALGO_SCRYPT = "scrypt";
 
-static const char* get_version_header(unsigned version)
+const char* get_version_header(unsigned version)
 {
     switch (version)
     {
@@ -711,10 +713,21 @@ private:
         "options"};
     TCLAP::UnlabeledValueArg<std::string> mount_point{
         "mount_point", "Mount point", true, "", "mount_point"};
-    TCLAP::SwitchArg case_insensitive{"i",
+    TCLAP::SwitchArg case_insensitive{"",
                                       "insensitive",
                                       "Converts the case of all filenames so "
                                       "that it works case insensitively"};
+    TCLAP::SwitchArg enable_nfc{
+        "",
+        "nfc",
+        "Normalizes all filenames to normal form composed (NFC). By default, enabled on macOS and "
+        "disabled on other platforms for compatibility of native applications.",
+#ifdef __APPLE__
+        true
+#else
+        false
+#endif
+    };
 
 public:
     void parse_cmdline(int argc, const char* const* argv) override
@@ -737,6 +750,7 @@ public:
         cmdline.add(&fuse_options);
         cmdline.add(&single_threaded);
         cmdline.add(&case_insensitive);
+        cmdline.add(&enable_nfc);
         cmdline.parse(argc, argv);
 
         if (pass.isSet() && !pass.getValue().empty())
@@ -860,7 +874,7 @@ public:
         }
         if (!background.getValue())
             fuse_args.push_back("-f");
-            
+
 #ifdef __APPLE__
         const char* copyfile_disable = ::getenv("COPYFILE_DISABLE");
         if (copyfile_disable)
@@ -874,8 +888,14 @@ public:
         }
 #elif _WIN32
         fuse_args.push_back("-ouid=-1,gid=-1");
+#else
+        fuse_args.push_back("-o");
+        fuse_args.push_back("big_writes");
 #endif
-
+        fuse_args.push_back("-o");
+        fuse_args.push_back("fsname=securefs");
+        fuse_args.push_back("-o");
+        fuse_args.push_back("subtype=securefs");
         if (fuse_options.isSet())
         {
             for (const std::string& opt : fuse_options.getValue())
@@ -908,8 +928,15 @@ public:
         if (insecure.getValue())
             fsopt.flags.value() |= kOptionNoAuthentication;
         if (case_insensitive.getValue())
+        {
+            INFO_LOG("Mounting as a case insensitive filesystem");
             fsopt.flags.value() |= kOptionCaseFoldFileName;
-
+        }
+        if (enable_nfc.getValue())
+        {
+            INFO_LOG("Mounting as a Unicode normalized filesystem");
+            fsopt.flags.value() |= kOptionNFCFileName;
+        }
         std::shared_ptr<FileStream> lock_stream;
         DEFER(if (lock_stream) {
             lock_stream->close();
@@ -1037,9 +1064,6 @@ static inline const char* true_or_false(bool v) { return v ? "true" : "false"; }
 
 class VersionCommand : public CommandBase
 {
-private:
-    const char* version_string = "0.8.3";
-
 public:
     void parse_cmdline(int argc, const char* const* argv) override
     {
@@ -1050,7 +1074,7 @@ public:
     int execute() override
     {
         using namespace CryptoPP;
-        printf("securefs %s\n", version_string);
+        printf("securefs %s\n", GIT_VERSION);
         printf("Crypto++ %g\n", CRYPTOPP_VERSION / 100.0);
 #ifdef WIN32
         HMODULE hd = GetModuleHandleW((sizeof(void*) == 8) ? L"winfsp-x64.dll" : L"winfsp-x86.dll");
@@ -1078,20 +1102,21 @@ public:
         printf("libfuse %d\n", fuse_version_func());
 #endif
 
+        printf("utf8proc %s\n", utf8proc_version());
+
 #ifdef CRYPTOPP_DISABLE_ASM
         fputs("\nBuilt without hardware acceleration\n", stdout);
 #else
 #if CRYPTOPP_BOOL_X86 || CRYPTOPP_BOOL_X32 || CRYPTOPP_BOOL_X64
-        fprintf(stdout,
-                "\nHardware features available:\nSSE2: %s\nSSE3: %s\nSSE4.1: %s\nSSE4.2: "
-                "%s\nAES-NI: %s\nCLMUL: %s\nSHA: %s\n",
-                HasSSE2() ? "true" : "false",
-                HasSSSE3() ? "true" : "false",
-                HasSSE41() ? "true" : "false",
-                HasSSE42() ? "true" : "false",
-                HasAESNI() ? "true" : "false",
-                HasCLMUL() ? "true" : "false",
-                HasSHA() ? "true" : "false");
+        printf("\nHardware features available:\nSSE2: %s\nSSE3: %s\nSSE4.1: %s\nSSE4.2: "
+               "%s\nAES-NI: %s\nCLMUL: %s\nSHA: %s\n",
+               HasSSE2() ? "true" : "false",
+               HasSSSE3() ? "true" : "false",
+               HasSSE41() ? "true" : "false",
+               HasSSE42() ? "true" : "false",
+               HasAESNI() ? "true" : "false",
+               HasCLMUL() ? "true" : "false",
+               HasSHA() ? "true" : "false");
 #endif
 #endif
         return 0;
@@ -1167,7 +1192,8 @@ public:
                format_version == 1 ? 4096 : config_json["block_size"].asUInt());
         printf("Content IV size: %u bits\n",
                format_version == 1 ? 256 : config_json["iv_size"].asUInt() * 8);
-        printf("Password derivation algorithm: PBKDF2-HMAC-SHA256\n");
+        printf("Password derivation algorithm: %s\n",
+               config_json.get("pbkdf", PBKDF_ALGO_PKCS5).asCString());
         printf("Password derivation iterations: %u\n", config_json["iterations"].asUInt());
         printf("Per file key generation algorithm: %s\n",
                format_version < 4 ? "HMAC-SHA256" : "AES");
@@ -1218,7 +1244,7 @@ int commands_main(int argc, const char* const* argv)
         for (std::unique_ptr<CommandBase>& command : cmds)
         {
             if (strcmp(argv[0], command->long_name()) == 0
-                || (argv[0] != 0 && argv[0][0] == command->short_name() && argv[0][1] == 0))
+                || (argv[0] != nullptr && argv[0][0] == command->short_name() && argv[0][1] == 0))
             {
                 command->parse_cmdline(argc, argv);
                 return command->execute();
