@@ -1,4 +1,5 @@
 #include "lite_operations.h"
+#include "apple_xattr_workaround.h"
 #include "lite_fs.h"
 #include "lite_stream.h"
 #include "logger.h"
@@ -92,10 +93,7 @@ namespace lite
 
     void* init(struct fuse_conn_info* fsinfo)
     {
-#ifdef FUSE_CAP_BIG_WRITES
-        fsinfo->want |= FUSE_CAP_BIG_WRITES;
-        fsinfo->max_write = static_cast<unsigned>(-1);
-#endif
+        (void)fsinfo;
 #ifdef FSP_FUSE_CAP_READDIR_PLUS
         fsinfo->want |= (fsinfo->capable & FSP_FUSE_CAP_READDIR_PLUS);
 #endif
@@ -483,21 +481,42 @@ namespace lite
 #ifdef __APPLE__
     int listxattr(const char* path, char* list, size_t size)
     {
+        TRACE_LOG("%s %s", __func__, path);
         auto filesystem = get_local_filesystem();
-        return static_cast<int>(filesystem->listxattr(path, list, size));
+        int rc = static_cast<int>(filesystem->listxattr(path, list, size));
+        transform_listxattr_result(list, size);
+        if (rc < 0)
+        {
+            TRACE_LOG("%s %s encountered error: %d (%s)",
+                      __func__,
+                      path,
+                      -rc,
+                      OSService::stringify_system_error(-rc).c_str());
+        }
+        return rc;
     }
 
     int getxattr(const char* path, const char* name, char* value, size_t size, uint32_t position)
     {
+        TRACE_LOG("%s %s with position=%u, name=%s", __func__, path, position, name);
         if (position != 0)
             return -EINVAL;
-        if (strcmp(name, "com.apple.quarantine") == 0)
-            return -ENOATTR;    // workaround for the "XXX is damaged" bug on OS X
-        if (strcmp(name, "com.apple.FinderInfo") == 0)
-            return -ENOATTR;    // stupid Apple hardcodes the size of xattr values
+        int rc = precheck_getxattr(&name);
+        if (rc <= 0)
+            return rc;
 
         auto filesystem = get_local_filesystem();
-        return static_cast<int>(filesystem->getxattr(path, name, value, size));
+        rc = static_cast<int>(filesystem->getxattr(path, name, value, size));
+        if (rc < 0)
+        {
+            TRACE_LOG("%s %s with name=%s encountered error: %d (%s)",
+                      __func__,
+                      path,
+                      name,
+                      -rc,
+                      OSService::stringify_system_error(-rc).c_str());
+        }
+        return rc;
     }
 
     int setxattr(const char* path,
@@ -507,22 +526,51 @@ namespace lite
                  int flags,
                  uint32_t position)
     {
+        TRACE_LOG("%s %s with position=%u, name=%s, value=%s",
+                  __func__,
+                  path,
+                  position,
+                  name,
+                  escape_nonprintable(value, size).c_str());
         if (position != 0)
             return -EINVAL;
-        if (strcmp(name, "com.apple.quarantine") == 0)
-            return 0;    // workaround for the "XXX is damaged" bug on OS X
-        if (strcmp(name, "com.apple.FinderInfo") == 0)
-            return -EACCES;    // stupid Apple hardcodes the size of xattr values
+        int rc = precheck_setxattr(&name, &flags);
+        if (rc <= 0)
+            return rc;
         if (!value || size == 0)
             return 0;
 
         auto filesystem = get_local_filesystem();
-        return filesystem->setxattr(path, name, const_cast<char*>(value), size, flags);
+        rc = filesystem->setxattr(path, name, const_cast<char*>(value), size, flags);
+        if (rc < 0)
+        {
+            TRACE_LOG("%s %s with name=%s encountered error: %d (%s)",
+                      __func__,
+                      path,
+                      name,
+                      -rc,
+                      OSService::stringify_system_error(-rc).c_str());
+        }
+        return rc;
     }
     int removexattr(const char* path, const char* name)
     {
+        TRACE_LOG("%s %s with name=%s", __func__, path, name);
+        int rc = precheck_removexattr(&name);
+        if (rc <= 0)
+            return rc;
         auto filesystem = get_local_filesystem();
-        return filesystem->removexattr(path, name);
+        rc = filesystem->removexattr(path, name);
+        if (rc < 0)
+        {
+            TRACE_LOG("%s %s with name=%s encountered error: %d (%s)",
+                      __func__,
+                      path,
+                      name,
+                      -rc,
+                      OSService::stringify_system_error(-rc).c_str());
+        }
+        return rc;
     }
 #endif
 
@@ -548,11 +596,13 @@ namespace lite
         opt->unlink = &::securefs::lite::unlink;
         opt->mkdir = &::securefs::lite::mkdir;
         opt->rmdir = &::securefs::lite::rmdir;
+#ifndef _WIN32
         opt->chmod = &::securefs::lite::chmod;
         opt->chown = &::securefs::lite::chown;
         opt->symlink = &::securefs::lite::symlink;
         opt->link = &::securefs::lite::link;
         opt->readlink = &::securefs::lite::readlink;
+#endif
         opt->rename = &::securefs::lite::rename;
         opt->fsync = &::securefs::lite::fsync;
         opt->utimens = &::securefs::lite::utimens;
